use std::{cell::RefCell, panic, rc::Rc};

use georgeemu::GeorgeEmu;
use minifb::{Scale, ScaleMode, Window, WindowOptions};
use wasm_bindgen::prelude::*;
use web_sys::HtmlElement;

fn window() -> web_sys::Window {
    web_sys::window().expect("no global `window` exists")
}

fn request_animation_frame(f: &Closure<dyn FnMut()>) {
    window()
        .request_animation_frame(f.as_ref().unchecked_ref())
        .expect("should register `requestAnimationFrame` OK");
}

const WIDTH: usize = 512;
const HEIGHT: usize = 380;

#[wasm_bindgen(start)]
pub fn main() {
    panic::set_hook(Box::new(console_error_panic_hook::hook));
    let rom = *include_bytes!("./demo.rom");
    let document = window().document().unwrap();
    let george_canvas: HtmlElement = document
        .get_element_by_id("george")
        .unwrap()
        .dyn_into::<HtmlElement>()
        .unwrap();
    let window = Window::new(
        george_canvas,
        WIDTH,
        HEIGHT,
        WindowOptions {
            resize: true,
            borderless: true,
            title: true,
            transparency: false,
            scale: Scale::FitScreen,
            scale_mode: ScaleMode::AspectRatioStretch,
            topmost: false,
            none: true,
        },
    )
    .unwrap();
    let mut emu = GeorgeEmu::builder().rom(rom).window(window).build();

    let f = Rc::new(RefCell::new(None));
    let g = f.clone();

    *g.borrow_mut() = Some(Closure::wrap(Box::new(move || {
        // cycle roughly however many cycles would've run during a frame
        for _ in 0..32000 {
            emu.cycle();
        }
        emu.draw();
        // schedule this closure for running again at next frame
        request_animation_frame(f.borrow().as_ref().unwrap());
    }) as Box<dyn FnMut() + 'static>));

    request_animation_frame(g.borrow().as_ref().unwrap());
}
